#!/usr/bin/python

dll_spec = [
    {
        "name" : "koretest",
        "description" : "provides 2 components",
        "components" : [
            {
                "name" : "Test Plugin",
                "type" : "Plugin",
                "class" : "TestPlugin",
                "class/fqn" : "test::TestPlugin",
                "factory" : "koretest_factory",
                "description" : "Just a dummy plugin for testing",
                "version" : "0.2.0",
                "version/major" : "0",
                "version/minor" : "2",
                "version/release" : "0",
                "api/version" : "0.2.0",
                "api/version/major" : "0",
                "api/version/minor" : "2",
                "api/version/release" : "0",
                "services" : [ "/kore/service/null", "/kore/service/none", "/kore/service/empty", "/test/service/test" ],
                "x-kodek/singleton" : "1"
            }
        ]
    }
]

def create_avp ( key, val ):
    return '{ "'+key+'", "'+val+'" }'

def create_dll ( dll_desc ):
    for dll in dll_desc:
        rez = '\nstatic const char* lib'+dll["name"]+'_metainfo[][2] = \n{\n'
        rez = rez + '    ' + create_avp( "/kore/library", '~'+dll["name"] ) + ',\n'
        rez = rez + '    ' + create_avp( '~'+dll["name"]+'/name', dll["name"] ) + ',\n'
        rez = rez + '    ' + create_avp( '~'+dll["name"]+'/description', dll["description"] ) + ',\n'
        for comp in dll["components"]:
            rez = rez + '    ' + create_avp( '~'+dll["name"]+'/component', '~'+comp["name"] ) + ',\n'
            for k in comp.keys():
                if k not in ["services", "x-kodek/singleton"]:
                    rez = rez + '    ' + create_avp( '~'+comp["name"]+'/'+k, comp[k] ) + ',\n'
            for srv in comp["services"]:
                rez = rez + '    ' + create_avp( '~'+comp["name"]+'/service', srv ) + ',\n'
        rez = rez + '    { 0, 0 }\n};\n'
        rez = rez + '\nkore::MetaInfo* '+dll["name"]+'_metainfo()\n'
        rez = rez + '{\n'
        rez = rez + '    static kore::MetaInfo* info = 0;\n'
        rez = rez + '    if( !info )\n'
        rez = rez + '        info = new kore::MetaInfo( lib'+dll['name']+'_metainfo );\n'
        rez = rez + '    return info;\n'
        rez = rez + '}\n'
        for comp in dll['components']:
            rez = rez + '\n'
            rez = rez + 'kore::Interface* '+comp['factory']+'()\n'
            rez = rez + '{\n'
            if comp['x-kodek/singleton'] == '0':
                rez = rez + '    return new '+comp['class/fqn']+'();\n'
            else:
                rez = rez + '    static '+comp['class/fqn']+'* inst = 0;\n'
                rez = rez + '    if( !inst )\n'
                rez = rez + '        inst = new '+comp['class/fqn']+'();\n'
                rez = rez + '    return inst;\n'
            rez = rez + '}\n'
    return rez

import sys

def readString ( prompt = '', default = None ):
    line = ''
    if default != None:
        sys.stderr.write( prompt+' ['+default+']: ' )
        line = sys.stdin.readline()
    else:
        while line == '' or line == '\n':
            sys.stderr.write( prompt+': ' )
            line = sys.stdin.readline()
    if line == '' or line == '\n':
        return default
    return line[:-1]

def component_wizard ():
    comp = {}
    sys.stderr.write('1.Mandatory info' )
    comp['name'] = readString( 'Component name' )
    comp['class'] = readString( 'Component class name', comp['name'] )
    comp['factory'] = readString( 'Component factory function', 'kore_'+comp['class']+'_factory' )
    sys.stderr.write('2.Recommended info')
    comp['services'] = []
    srv = readString( 'Add service', '' )
    while srv != '':
        comp['services'] = comp['services'] + [srv]
        srv = readString( 'Add service', '' )
    comp['description'] = readString( 'Component description', '' )
    comp['class/fqn'] = readString( 'Class fully qualified name', comp['class'] )
    sys.stderr.write('3.Optional info')
    comp['version'] = readString( 'Component version (major.minor.release)', '0.0.1' )
    comp['version/major'] = readString( 'Component major version', '0' )
    comp['version/minor'] = readString( 'Component major version', '0' )
    comp['version/release'] = readString( 'Component release number', '1' )
    comp['api/version'] = readString( 'Korelib version (major.minor.release)', '0.2.0' )
    comp['api/version/major'] = readString( 'Korelib major version', '0' )
    comp['api/version/minor'] = readString( 'Korelib major version', '2' )
    comp['api/version/release'] = readString( 'Korelib release number', '0' )
    sys.stderr.write('4.Kodek-specific info')
    comp['x-kodek/singleton'] = str(readString( 'Is this component a singleton ?', 'n' )[0] != 'n')
    return comp

def dll_wizard ():
    dll = {}
    dll["name"] = readString( 'Library name' )
    dll["description"] = readString( 'Library description', dll["name"] )
    more = readString( 'Do you want to add a component ?', 'y' )[0] == 'y'
    dll["components"] = []
    while more:
        dll["components"] = dll["components"] + [component_wizard()]
        more = readString( 'Do you want to add one more component ?', 'n' )[0] != 'n'
    return dll

#dll_wizard()
print create_dll( [dll_wizard()] )
