#include <iostream>
//#include  <strstream>
#include <sstream>

#include <kore/componentproxy.h>
#include <kore/metainfo.h>
#include <kore/componenttrader.h>
#include <kore/componentref.h>
#include <kore/libraryloader.h>
#include <kore/library.h>
#include <kore/librarylocator.h>

using namespace kore;
using namespace std;

#ifdef KORE_WIN32
// cl.exe+stlport hack: cl.exe hates 'endl' and borks the compilation
#define endl "\n"
#endif

typedef Interface* (*instance_fcn_type)( void );

class kore::ComponentProxyPrivate
{
public:
    ComponentProxyPrivate()
    {
        instance = 0;
        info = 0;
    }
    
    Interface* instance;
    MetaInfo* info;
};

ComponentProxy::ComponentProxy()
{
    cerr << "ComponentProxy(): " << this << endl;
    _d = new ComponentProxyPrivate();
}

ComponentProxy::ComponentProxy( MetaInfo* info )
{
    cerr << "ComponentProxy( MetaInfo: " << info << " ): " << this << endl;
    _d = new ComponentProxyPrivate();
    _d->info = info;
}

ComponentProxy::ComponentProxy( MetaInfo* info, Interface* inst )
{
    cerr << "ComponentProxy( MetaInfo: " << info << ", Interface: " << inst << " ): " << this << endl;
    _d = new ComponentProxyPrivate();
    _d->info = info;
    setInstance( inst );
}

ComponentProxy::~ComponentProxy()
{
    cerr << "~ComponentProxy(): " << this << endl;
    delete _d;
}

Interface* ComponentProxy::instance()
{
    if( !_d->info )
    {
        cerr << "No Info..." << endl;
        if( !_d->instance )
            return 0;
    }
    if( !_d->instance )
    {
        cerr << "No instance..." << endl;
        ComponentRef tref = ComponentTrader::traderRef();
        if( tref.instance() )
        {
            ComponentTrader* trader = dynamic_cast<ComponentTrader*>(tref.instance());
            tref = trader->registeredComponent("/kore/service/loader");
            if( tref.instance() )
            {
                LibraryLoader* loader = dynamic_cast<LibraryLoader*>(tref.instance());
                Library* lib = loader->loadLibrary( _d->info->find("library/name"), _d->info->find("library/path") );
                if( lib )
                {
                    void* addr = lib->symbolAddress( _d->info->find("factory") );
                    if( addr )
                    {
                        instance_fcn_type factory = (instance_fcn_type)addr;
                        _d->instance = factory();
                        _d->info->insert( "id", trader->idAsString( _d->instance ) );
                        trader->registerService( "/kore/service/id", ComponentRef( this ) );
                        trader->registerService( trader->idAsService( _d->instance ), ComponentRef( this ) );
                    }
                    else
                    {
                        cerr << "Warning[kore]: couldn't find the component factory: "<< _d->info->find("factory") << endl;
                        cerr << "Hint[kore]: check if the factory exists and is declared extern \"C\"" << endl;
                    }
                }
            }
        }
    }
    return _d->instance;
}

void ComponentProxy::setInstance( Interface* inst )
{
    _d->instance = inst;
}

Interface* ComponentProxy::newInstance()
{
    ComponentRef tref = ComponentTrader::traderRef();
    if( tref.instance() )
    {
        ComponentTrader* trader = dynamic_cast<ComponentTrader*>(tref.instance());
        const char* fqn = _d->info->find("class/fqn");
        char* tmp = new char[24+strlen(fqn)+1];
        strcpy(tmp,"/kore/metainfo_template/");
        strcat(tmp,fqn);
        tref = trader->registeredComponent(tmp);
        delete tmp;
        MetaInfo* nfo = new MetaInfo( tref.metaInfo()->elements() );
        ComponentProxy* proxy = new ComponentProxy( nfo );
        // check for singleton
        //if( proxy->instance() == _d->instance )
        //{
        //    return _d->instance;
        //}
        trader->registerComponent( ComponentRef( proxy ) );
        return proxy->instance();
    }
    else
        return 0;
}

MetaInfo* ComponentProxy::metaInfo()
{
    return _d->info;
}
