
#include <kore/componenttrader.h>
#include <kore/metainfo.h>
#include <kore/libraryloader.h>
#include <kore/library.h>

#include <typeinfo>
#include <strstream>
//#include <sstream>

//#ifdef _WIN32
#include "config.h"
#ifdef EXT_HASH_MAP
#include <ext/hash_map>
#else
#include <hash_map>
#endif
#include <vector>
#include <set>
//#else
//#include <hash_map>
//#include <set>
//#endif

using namespace kore;
using namespace std;
#include "config.h"
#ifdef EXT_HASH_MAP
using namespace __gnu_cxx;
#endif

#ifdef KORE_WIN32
// cl.exe+stlport hack: cl.exe hates 'endl' and borks the compilation
#define endl "\n"
#define ends '\0'
#endif

//#ifdef _WIN32
//    struct ltstr
//    {
//        bool operator()(const char* s1, const char* s2) const
//        {
//            return strcmp(s1,s2) == -1;
//        }
 //   };
    //Windows STL doesn't have stl::hash_multimap
//    typedef hash_multimap<const char*,const ComponentRef, ltstr> srv_hash_type;

//#else
    struct eqstr
    {
        bool operator()(const char* s1, const char* s2) const
        {
            return strcmp(s1,s2) == 0;
        }
    };
    typedef hash_multimap<const char*,const ComponentRef, hash<const char*>, eqstr> srv_hash_type;
//#endif
struct ltptr
{
    bool operator()(const ComponentRef& sp1,const ComponentRef& sp2) const
    {
        return &sp1 < &sp2;
    }
};
typedef set<const ComponentRef , ltptr> sp_set_type;

class kore::ComponentTraderPrivate
{
public:
    ComponentTraderPrivate()
    {
    }
    ~ComponentTraderPrivate()
    {
    }
    
    // the list (multimap<key: service name, value: Service>) of Services
    // registered to this ServiceManager
    srv_hash_type services;
    static ComponentRef traderRef;
    static Interface* instance;
};

ComponentTrader::ComponentTrader()
{
    _d = new ComponentTraderPrivate();
    //cout << "ComponentTrader()" << endl;
}
ComponentTrader::~ComponentTrader()
{
    unregisterComponents();
    //cout << "~ComponentTrader()" << endl;
    delete _d;
}
/*
void ComponentTrader::registerService(const ServiceProvider::Service* srv)
{
    if( !srv->provider() )
        return;
    srv->provider()->registeringService(this, srv);
    _services.insert(srv_hash_type::value_type(srv->name(),srv));
    srv->provider()->serviceRegistered(this, srv);
}

void ComponentTrader::unregisterService(const ServiceProvider::Service* srv)
{
    pair<srv_hash_type::iterator, srv_hash_type::iterator> p = _services.equal_range(srv->name());
    vector<srv_hash_type::iterator> rm;
    for( srv_hash_type::iterator i = p.first; i != p.second; ++i )
        if( i->second == srv )
            rm.push_back(i);
    int n = rm.size();
    for(int j = 0; j<n; j++)
    {
        srv->provider()->unregisteringService(this, srv);
        _services.erase(rm[j]);
        srv->provider()->serviceUnregistered(this, srv);
    }
}
*/
void ComponentTrader::registerComponent( const ComponentRef& comp )
{
    //cerr << "Registering component..." << endl;
    //comp.metaInfo()->ddump();
    const char** srvs = comp.metaInfo()->findAll("service");
    for(int i=0; srvs[i]; i++)
    {
        //cerr << "Registering service: " << srvs[i] << endl;
        _d->services.insert(srv_hash_type::value_type(srvs[i],comp));
    }
    delete[] srvs;
}

void ComponentTrader::unregisterComponent(const ComponentRef& comp)
{
    const char** srvs = comp.metaInfo()->findAll("service");
    const char* name = comp.metaInfo()->find("name");
    for(int i=0; srvs[i]; i++)
    {
        pair<srv_hash_type::iterator, srv_hash_type::iterator> p = _d->services.equal_range(srvs[i]);
        vector<srv_hash_type::iterator> rm;
        for( srv_hash_type::iterator i = p.first; i != p.second; ++i )
            if( !strcmp(i->second.metaInfo()->find("name"),name) )
                rm.push_back(i);
        int n = rm.size();
        for(int j = 0; j<n; j++)
        {
            _d->services.erase(rm[j]);
        }
    }
    delete[] srvs;
}

const char** ComponentTrader::registeredServices() const
{
    int n = _d->services.size();
    const char** srvs = new const char*[n+1];
    srv_hash_type::const_iterator b,e;
    b = _d->services.begin();
    e = _d->services.end();
    int i = 0;
    for( srv_hash_type::const_iterator j = b; j != e; ++j, ++i )
        srvs[i] = j->first;
    srvs[i] = 0;
    return srvs;
}
/*
const char** ComponentTrader::registeredServices(const ComponentRef& comp) const
{
    const char* name = comp.metaInfo()->find("kore/component/name");
    vector<const char*> srvs;
    srv_hash_type::const_iterator b,e;
    b = _services.begin();
    e = _services.end();
    for( srv_hash_type::const_iterator j = b; j != e; ++j )
        if( !strcmp(i->second.metaInfo()->find("kore/component/name"),name) )
            srvs.push_back( j->first );
    int n = srvs.size();
    const char** res = new const char*[n+1];
    int i;
    for( i= 0; i < n; i++ )
        res[i] = srvs[i];
    res[i] = 0;
    return res;
}
*/

ComponentRef ComponentTrader::registeredComponent(const char* service) const
{
    cerr << "Looking for service: " << service << endl;
    pair<srv_hash_type::const_iterator, srv_hash_type::const_iterator> p = _d->services.equal_range(service);
    for( srv_hash_type::const_iterator j = p.first; j != p.second; ++j)
        return j->second;
    cout << "No registered component found..." << endl;
    return ComponentRef();
}

ComponentRef* ComponentTrader::registeredComponents(const char* service) const
{
    int n = _d->services.count(service);
    ComponentRef* srvs = new ComponentRef[n+1];
    pair<srv_hash_type::const_iterator, srv_hash_type::const_iterator> p = _d->services.equal_range(service);
    int i = 0;
    for( srv_hash_type::const_iterator j = p.first; j != p.second; ++j, ++i )
        srvs[i] = j->second;
    srvs[i] = ComponentRef();
    return srvs;
}

ComponentRef* ComponentTrader::registeredComponents() const
{
    int n = _d->services.size();
    //cerr << n << " registered service/component pairs" << endl;
    vector<const ComponentRef*> tmp;
    srv_hash_type::const_iterator b,e;
    b = _d->services.begin();
    e = _d->services.end();
    int i = 0;
    for( srv_hash_type::const_iterator j = b; j != e && i <= n ; ++j, ++i)
        if( j->second.isValid() )
        {
            //cerr << j->second.metaInfo()->find("name") << endl;
            int n1 = tmp.size();
            //srvs[i++] = j->second;
            bool duplicate = false;
            for( int k = 0; k < n1; k ++ )
            {
                if( tmp[k]->proxy() == j->second.proxy() )
                {
                    duplicate = true;
                    break;
                }
            }
            if( !duplicate )
                tmp.push_back( &(j->second) );
        }
        else
        {
            cerr << "Warning[kore]: found invalid component reference in Component Trader" << endl;
            abort();
        }
    n = tmp.size();
    //cerr << n << " unique components" << endl;
    ComponentRef* srvs = new ComponentRef[n+1];
    for( i = 0; i < n; i++ )
        srvs[i] = *(tmp[i]);
    srvs[i] = ComponentRef();
    return srvs;
/*
    sp_set_type sps;
    const Service** srvs = registeredServices();
	int i = 0;
    for(i = 0; srvs[i]; i++)
        sps.insert(srvs[i]->provider());
    delete[] srvs;
    int n = sps.size();
    ServiceProvider** res = new ServiceProvider*[n+1];
    i = 0;
    sp_set_type::const_iterator b,e;
    b = sps.begin();
    e = sps.end();
    for( sp_set_type::const_iterator j = b; j != e; ++j, ++i )
        res[i] = *j;
    res[i] = 0;
    return res;
*/
}

void ComponentTrader::registerComponents(const ComponentRef* comps)
{
    for( int i = 0; &comps[i]; i++ )
        registerComponent(comps[i]);
}
void ComponentTrader::unregisterComponents()
{
    ComponentRef* srvs = registeredComponents();
    for(int i = 0; &srvs[i]; i++)
        unregisterComponent(srvs[i]);
    delete[] srvs;
}

ComponentRef ComponentTrader::traderRef()
{
    if( ComponentTraderPrivate::traderRef.instance() == 0 )
    {
        // time to bootstrap
        ComponentTrader* trader = dynamic_cast<ComponentTrader*>(kore_trader_factory());
        LibraryLoader* loader = dynamic_cast<LibraryLoader*>(kore_loader_factory());
	cerr << "trying to load kore" << endl;
        Library* lib = loader->loadLibrary( "kore" );
        if( lib )
        {
            cerr << "Library OK!" << endl;

            char* tmp = new char[100];
            strcpy( tmp, lib->name() );
            strcat( tmp, "_metainfo" );
            cerr << "Symbol: " << tmp << endl;
            void* sym = lib->symbolAddress(tmp);
            if( sym )
            {
                cerr << "Symbol OK!" << endl;
                typedef MetaInfo* (*info)();
                info fct = (info)sym;
                MetaInfo* i = fct();
                //i->ddump();
                // start parsing the meta info
                const char** rez = i->findAll("/kore/library");
                // for each library... ?
                for( int j = 0; rez[j]; j++)
                {
                    //cout << "Library: " << rez[j] << endl;
                    MetaInfo* libInfo = new MetaInfo();
                    char* tmp2 = new char[100];
                    strcpy( tmp2, rez[j] );
                    strcat( tmp2, "/name" );
                    const char* rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-name: " << rez2 << endl;
                        libInfo->insert( "name", strdup(rez2) );
                    }
                    strcpy( tmp2, rez[j] );
                    strcat( tmp2, "/description" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-desc: " << rez2 << endl;
                        libInfo->insert( "description", strdup(rez2) );
                    }
                    strcpy( tmp2, rez[j] );
                    strcat( tmp2, "/component" );
                    const char** rez3 = i->findAll( tmp2 );
                    // for each component...
                    for( int k = 0; rez3[k]; k++ )
                    {
                        //cout << endl << "Component: " << rez3[k] << endl;
                        MetaInfo* compInfo = new MetaInfo();
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/name" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-name: " << rez2 << endl;
                            compInfo->insert( "name", strdup(rez2) );
                            libInfo->insert( "component", strdup( rez2 ) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/type" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-type: " << rez2 << endl;
                            compInfo->insert( "type", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/class" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-class: " << rez2 << endl;
                            compInfo->insert( "class", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/class/fqn" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-class/fqn: " << rez2 << endl;
                            compInfo->insert( "class/fqn", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/factory" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-factory: " << rez2 << endl;
                            compInfo->insert( "factory", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/description" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-description: " << rez2 << endl;
                            compInfo->insert( "description", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/version" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-version: " << rez2 << endl;
                            compInfo->insert( "version", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/version/major" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-version/major: " << rez2 << endl;
                            compInfo->insert( "version/major", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/version/minor" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-version/minor: " << rez2 << endl;
                            compInfo->insert( "version/minor", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/version/release" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-version/release: " << rez2 << endl;
                            compInfo->insert( "version/release", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/api/version" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-api/version: " << rez2 << endl;
                            compInfo->insert( "api/version", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/api/version/major" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-api/version/major: " << rez2 << endl;
                            compInfo->insert( "api/version/major", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/api/version/minor" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-api/version/minor: " << rez2 << endl;
                            compInfo->insert( "api/version/minor", strdup(rez2) );
                        }
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/api/version/release" );
                        rez2 = i->find( tmp2 );
                        if( rez2 )
                        {
                            //cout << "-api/version/release: " << rez2 << endl;
                            compInfo->insert( "api/version/release", strdup(rez2) );
                        }
                        if( lib->name() )
                            compInfo->insert( "library/name", strdup(lib->name()) );
                        if( lib->path() )
                            compInfo->insert( "library/path", strdup(lib->path()) );
                        strcpy( tmp2, rez3[k] );
                        strcat( tmp2, "/service" );
                        const char** rez4 = i->findAll( tmp2 );
                        for( int l = 0; rez4[l]; l++ )
                        {
                            //cout << "Service: " << rez4[l] << endl;
                            compInfo->insert( "service", strdup( rez4[l] ) );
                        }
                        //cout << "-------------------------------" << endl;
                        //compInfo->ddump();
                        ComponentProxy* proxy;
                        // hack
                        if( !strcmp( "Trader", compInfo->find("type") ) )
                        {
                            proxy = new ComponentProxy( compInfo, trader );
                            ComponentTraderPrivate::traderRef = ComponentRef( proxy );
                            compInfo->insert( "id", trader->idAsString( proxy->instance() ) );
                            trader->registerService( "/kore/service/id", ComponentRef( proxy ) );
                            trader->registerService( trader->idAsService( proxy->instance() ), ComponentRef( proxy ) );
                        }
                        else if( !strcmp( "Loader", compInfo->find("type") ) )
                        {
                            proxy = new ComponentProxy( compInfo, loader );
                            compInfo->insert( "id", trader->idAsString( proxy->instance() ) );
                            trader->registerService( "/kore/service/id", ComponentRef( proxy ) );
                            trader->registerService( trader->idAsService( proxy->instance() ), ComponentRef( proxy ) );
                        }
                        else if( !strcmp( "Locator", compInfo->find("type") ) )
                        {
                            proxy = new ComponentProxy( compInfo, kore_locator_factory() );
                            compInfo->insert( "id", trader->idAsString( proxy->instance() ) );
                            trader->registerService( "/kore/service/id", ComponentRef( proxy ) );
                            trader->registerService( trader->idAsService( proxy->instance() ), ComponentRef( proxy ) );
                        }
                        else
                            proxy = new ComponentProxy( compInfo );
                        trader->registerComponent( ComponentRef( proxy ) );
                        // the "class" metainfo - needed for ComponentRef::newInstance()
                        MetaInfo* classInfo = new MetaInfo( compInfo->elements() );
                        proxy = new ComponentProxy( classInfo );
                        const char* fqn = compInfo->find("class/fqn");
                        char* tmp = new char[24+strlen(fqn)+1];
                        strcpy(tmp,"/kore/metainfo_template/");
                        strcat(tmp,fqn);
                        trader->registerService( tmp, ComponentRef( proxy ) );
                    }
                    //libInfo->ddump();
                    delete[] tmp2;
                }
                delete[] rez;
                // end parsing metainfo
                delete i;
            }
            else
            {
                cerr << "Warning[kore]: could not find kore_metainfo!" << endl;
                cerr << loader->lastError() << endl;
            }
            delete tmp;
        }
        else
        {
            cerr << "Warning[kore]: " << loader->libName2fileName( "kore" ) << " not found!" << endl;
        }
    }
    return ComponentTraderPrivate::traderRef;
}

unsigned long ComponentTrader::idAsLong( void* id ) const
{
    return (unsigned long) id;
}

unsigned long ComponentTrader::idAsLong( Interface* id ) const
{
    return (unsigned long) id;
}

const char* ComponentTrader::idAsString( void* id ) const
{
    ostrstream ostr;
    ostr << id << ends;
    char* idbuff = new char[ostr.pcount()+1];
    strcpy( idbuff, ostr.str() );
    return idbuff;
}

const char* ComponentTrader::idAsString( Interface* id ) const
{
    ostrstream ostr;
    ostr << (void*)id << ends;
    char* idbuff = new char[ostr.pcount()+1];
    strcpy( idbuff, ostr.str() );
    return idbuff;
}

const char* ComponentTrader::idAsService( void* id ) const
{
    char* strid = const_cast<char*>( idAsString( id ) );
    char* prefix = "/kore/service/id/";
    char* srvid = new char[strlen(prefix)+strlen(strid)+1];
    strcpy( srvid, prefix );
    strcat( srvid, strid );
    delete strid;
    delete prefix;
    return srvid;
}

const char* ComponentTrader::idAsService( Interface* id ) const
{
    char* strid = const_cast<char*>( idAsString( id ) );
    char* prefix = "/kore/service/id/";
    char* srvid = new char[strlen(prefix)+strlen(strid)+1];
    strcpy( srvid, prefix );
    strcat( srvid, strid );
    delete strid;
  //  delete prefix;
    return srvid;
}

ComponentRef ComponentTrader::idAsRef( void* id ) const
{
    return registeredComponent( idAsService( id ) );
}

ComponentRef ComponentTrader::idAsRef( Interface* id ) const
{
    return registeredComponent( idAsService( id ) );
}

void ComponentTrader::registerService( const char* service, const ComponentRef& provider )
{
    _d->services.insert(srv_hash_type::value_type(service,provider));
}

// !! component identification should be done by comparing the ComponentProxy pointers
void ComponentTrader::unregisterService( const char* service, const ComponentRef& provider )
{
    const char* name = provider.metaInfo()->find("name");
    pair<srv_hash_type::iterator, srv_hash_type::iterator> p = _d->services.equal_range(service);
    vector<srv_hash_type::iterator> rm;
    for( srv_hash_type::iterator i = p.first; i != p.second; ++i )
        if( !strcmp(i->second.metaInfo()->find("name"),name) )
            rm.push_back(i);
    int n = rm.size();
    for(int j = 0; j<n; j++)
    {
        _d->services.erase(rm[j]);
    }
}

ComponentRef ComponentTraderPrivate::traderRef;
Interface* ComponentTraderPrivate::instance = 0;
