
#include <kore/library.h>
#include <kore/metainfo.h>
#ifndef _WIN32
#include <dlfcn.h>
#endif

using namespace kore;
using namespace std;


class kore::LibraryPrivate
{
public:
    LibraryPrivate()
    {
        info = new MetaInfo();
        libname = 0;
        libpath = 0;
        flags = 0;
    }
    ~LibraryPrivate()
    {
    }

    HMODULE handle;
    MetaInfo* info;
    const char* libname;
    const char* libpath;
    int flags;
};

Library::Library()
{
    _d = new LibraryPrivate();
}

Library::Library(HMODULE handle, const char* libname, const char* libpath, int flags)
{
    _d = new LibraryPrivate();
    _d->handle = handle;
    _d->libname = libname;
    _d->libpath = libpath;
    _d->flags = flags;
}

Library::~Library()
{
#if defined( KORE_WIN32 )
	FreeLibrary( _d->handle );
#elif defined( KORE_BEOS )
    unload_add_on( _d->handle );
#elif defined( KORE_ATHEOS )
    unload_library( _d->handle );
#else
    dlclose( _d->handle );
#endif
    
    delete _d;
}

HMODULE Library::libHandle() const
{
    return _d->handle;
}

const char* Library::name() const
{
    return _d->libname;
}

const char* Library::path() const
{
    return _d->libpath;
}

void* Library::symbolAddress( const char* name )
{
	void* sym = 0;
#if defined( KORE_WIN32 )
	sym = GetProcAddress( _d->handle, name );
#elif defined( KORE_BEOS )
    if( B_OK != get_image_symbol( _d->handle, name, B_SYMBOL_TYPE_TEXT, &sym ) )
        sym = 0;
#elif defined( KORE_ATHEOS )
    if( 0 != get_symbol_address( _d->handle, name, -1, &sym ) )
	    sym = 0;
#else
    sym = dlsym( _d->handle, name );
#endif
    return sym;
}

