
#include <kore/libraryloader.h>
#include <kore/library.h>
#ifndef _WIN32
#include <dlfcn.h>
#endif
#include <string>

using namespace kore;
using namespace std;


class kore::LibraryLoaderPrivate
{
public:
    LibraryLoaderPrivate()
    {
        lastError = new char[101];
    }
    ~LibraryLoaderPrivate()
    {
        delete lastError;
    }

    char* lastError;
};

LibraryLoader::LibraryLoader()
{
    _d = new LibraryLoaderPrivate();
}

LibraryLoader::~LibraryLoader()
{
    delete _d;
}

Library* LibraryLoader::loadLibrary(const char* libName, const char* libPath, int libFlags)
{
    HMODULE handle;
    char* fileName = libName2fileName(libName);
    char* filePath = 0;
    Library* res = 0;
    if( libPath && libPath[0] )
    {
        int n = strlen(libPath);
        int slash = libPath[n-1] == '/' ? 0 : 1;
        filePath = new char[n+strlen(fileName)+slash+1];
        strcpy(filePath, libPath);
        if( slash )
        {
            filePath[n] = '/';
            filePath[n+1] = 0;
        }
    }
    else
    {
        filePath = new char[strlen(fileName)+1];
        filePath[0] = 0;
    }
    strcat( filePath, fileName );
    //cout << "Loading: " << filePath << endl;
    delete[] fileName;
#if defined( KORE_WIN32 )
	handle = LoadLibrary(filePath);
#elif defined( KORE_BEOS )
    handle = load_add_on(filePath);
#elif defined( KORE_ATHEOS )
    handle = load_library( filePath, 0 );
#else
    int flags = 0;
    if( libFlags & LAZY )
        flags |= RTLD_LAZY;
    if( libFlags & NOW )
        flags |= RTLD_NOW;
    if( libFlags & GLOBAL )
        flags |= RTLD_GLOBAL;
    //
    if( !(flags & (RTLD_LAZY | RTLD_NOW)) )
        flags |= RTLD_LAZY;
    handle = dlopen(filePath, flags );
#endif
    delete[] filePath;
    //cout << "Loaded: " << handle << endl;
#if defined( KORE_BEOS )
    if( !(handle < B_NO_ERROR) )
#elif defined( KORE_ATHEOS )
    if( !(handle < 0) )
#else
    if( handle )
#endif
        res = new Library(handle,libName,libPath,libFlags);
    return res;
}

// "my_plugin" -> "libmy_plugin.so"/"my_plugin.dll"
char* LibraryLoader::libName2fileName(const char* libName)
{
    int n = strlen(libName);
#ifdef KORE_WIN32
	//char* fn = new char[n+1+4];
	//return strcat(strcpy(fn,libName),".dll");
	char* fn = new char[n+1+7];
	return strcat(strcat(strcpy(fn,"lib"),libName),".dll");
#else
    char* fn = new char[n+1+6];
    return strcat(strcat(strcpy(fn,"lib"),libName),".so");
#endif
}
// "libmy_plugin.so"/"my_plugin.dll" -> "my_plugin"
char* LibraryLoader::fileName2libName(const char* fileName)
{
#ifdef KORE_WIN32
    //int n = strlen(fileName) - 4;
	//char* ln = new char[n+1];
	char* pos = strstr(fileName, ".dll");
    int n = pos - fileName - 3;
    char* ln = new char[n+1];
    fileName += 3;
#else
    char* pos = strstr(fileName, ".so");
    int n = pos - fileName - 3;
    char* ln = new char[n+1];
    fileName += 3;
#endif
    strncpy(ln, fileName, n);
    ln[n] = 0;
    return ln;
}

const char* LibraryLoader::lastError()
{
    _d->lastError[0]=0;
#if defined( KORE_WIN32 )
    FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, 0,
        GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), _d->lastError, 100, NULL );
#elif defined( KORE_BEOS )
    strncpy(_d->lastError, strerror(errno), 100);
#elif defined( KORE_ATHEOS )
    strncpy(_d->lastError, strerror(errno), 100);
#else
    strncpy(_d->lastError, dlerror(), 100);
#endif
    return _d->lastError;
}

