
#include <kore/librarylocator.h>
#include <kore/library.h>
#include <kore/libraryloader.h>
#include <kore/componenttrader.h>
#include <kore/componentref.h>
#include <kore/interface.h>
#include <kore/metainfo.h>
#include <string.h>
#ifndef _WIN32
#include <dirent.h>
#else
#include <io.h>
#endif
#include <sys/stat.h>

#include <iostream>

using namespace kore;
using namespace std;

#ifdef KORE_WIN32
// cl.exe+stlport hack: cl.exe hates 'endl' and borks the compilation
#define endl "\n"
#endif

LibraryLocator::LibraryLocator()
{
}

LibraryLocator::~LibraryLocator()
{
}

void LibraryLocator::scanLibrary( const char* name, const char* path, int flags )
{
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
        cerr << "Trader found" << endl;
        ComponentRef ref = trader->registeredComponent( "/kore/service/loader" );
        LibraryLoader* loader = dynamic_cast<LibraryLoader*>( ref.instance() );
        if( loader )
        {
            cerr << "Loader found" << endl;
            Library* lib = loader->loadLibrary( name, path, flags );
            if( path )
                cerr << "path: " << path << endl;
            else
                cerr << "default path" << endl;
	    cerr << "library: " << loader->libName2fileName( name ) << endl;
	    if( lib )
            {
                cerr << "Library OK!" << endl;
                scanLibrary( lib );
		cerr << "scanLib() ended" << endl;
                //delete lib;
            }
            else
            {
	        cerr << "Warning[kore]: could not load " << loader->libName2fileName( name ) << " . Trying the Python locator." << endl;
                cerr << loader->lastError() << endl;
                // try the python locator...
                LibraryLocator* pyLocator = dynamic_cast<LibraryLocator*>(trader->registeredComponent("/kore/service/locator/python").instance());
                if( pyLocator )
                {
                    pyLocator->scanLibrary( name, path, flags );
                }
                else
                    cerr << "Python locator not found" << endl;
            }
        }
    }
    else
        cerr << "Warning[kore]: ComponentTrader not found during LibraryLocator::scanLibrary( " << name << " )" << endl;
}

void LibraryLocator::scanLibrary( Library* lib )
{
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
        char* tmp = new char[100];
        strcpy( tmp, lib->name() );
        strcat( tmp, "_metainfo" );
        //cout << "Symbol: " << tmp << endl;
        void* sym = lib->symbolAddress(tmp);
        if( sym )
        {
            //cout << "Symbol OK!" << endl;
            typedef MetaInfo* (*info)();
            info fct = (info)sym;
            MetaInfo* i = fct();
            //cout << "Start ddump()" << endl;
            //i->ddump();
            //cout << "Done ddump()" << endl;
            // start parsing the meta info
            const char** rez = i->findAll("/kore/library");
            //cout << "Done findAll()" << endl;
            // for each library... ?
            for( int j = 0; rez[j]; j++)
            {
                //cout << "Library: " << rez[j] << endl;
                MetaInfo* libInfo = new MetaInfo();
                char* tmp2 = new char[100];
                strcpy( tmp2, rez[j] );
                strcat( tmp2, "/name" );
                const char* rez2 = i->find( tmp2 );
                if( rez2 )
                {
                    //cout << "-name: " << rez2 << endl;
                    char* tmp10 = new char[100];
                    strcpy(tmp10, "/kore/service/library/");
                    strcat(tmp10, rez2);
                    // was this library loaded before ?
                    if( trader->registeredComponent(tmp10).isValid() )
                    {
                        //fixme: this is a little bit too brutal
                        cerr << "Info[kore]: Library already registered: " << rez2 << endl;
                        return;
                    }
                    libInfo->insert( "name", strdup(rez2) );
                    libInfo->insert( "service", tmp10 );
                    libInfo->insert( "service", "/kore/service/library" );
                }
                strcpy( tmp2, rez[j] );
                strcat( tmp2, "/description" );
                rez2 = i->find( tmp2 );
                if( rez2 )
                {
                    //cout << "-desc: " << rez2 << endl;
                    libInfo->insert( "description", strdup(rez2) );
                }
                strcpy( tmp2, rez[j] );
                strcat( tmp2, "/component" );
                const char** rez3 = i->findAll( tmp2 );
                // for each component...
                for( int k = 0; rez3[k]; k++ )
                {
                    //cout << endl << "Component: " << rez3[k] << endl;
                    MetaInfo* compInfo = new MetaInfo();
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/name" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-name: " << rez2 << endl;
                        compInfo->insert( "name", strdup(rez2) );
                        libInfo->insert( "component", strdup( rez2 ) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/type" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-type: " << rez2 << endl;
                        compInfo->insert( "type", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/class" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-class: " << rez2 << endl;
                        compInfo->insert( "class", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/class/fqn" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-class/fqn: " << rez2 << endl;
                        compInfo->insert( "class/fqn", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/factory" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-factory: " << rez2 << endl;
                        compInfo->insert( "factory", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/description" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-description: " << rez2 << endl;
                        compInfo->insert( "description", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/version" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-version: " << rez2 << endl;
                        compInfo->insert( "version", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/version/major" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-version/major: " << rez2 << endl;
                        compInfo->insert( "version/major", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/version/minor" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-version/minor: " << rez2 << endl;
                        compInfo->insert( "version/minor", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/version/release" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-version/release: " << rez2 << endl;
                        compInfo->insert( "version/release", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/api/version" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-api/version: " << rez2 << endl;
                        compInfo->insert( "api/version", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/api/version/major" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-api/version/major: " << rez2 << endl;
                        compInfo->insert( "api/version/major", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/api/version/minor" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-api/version/minor: " << rez2 << endl;
                        compInfo->insert( "api/version/minor", strdup(rez2) );
                    }
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/api/version/release" );
                    rez2 = i->find( tmp2 );
                    if( rez2 )
                    {
                        //cout << "-api/version/release: " << rez2 << endl;
                        compInfo->insert( "api/version/release", strdup(rez2) );
                    }
                    if( lib->name() )
                        compInfo->insert( "library/name", strdup(lib->name()) );
                    if( lib->path() )
                        compInfo->insert( "library/path", strdup(lib->path()) );
                    strcpy( tmp2, rez3[k] );
                    strcat( tmp2, "/service" );
                    const char** rez4 = i->findAll( tmp2 );
                    for( int l = 0; rez4[l]; l++ )
                    {
                        //cout << "Service: " << rez4[l] << endl;
                        compInfo->insert( "service", strdup( rez4[l] ) );
                    }
                    //cout << "-------------------------------" << endl;
                    //compInfo->ddump();
                    ComponentProxy* proxy = new ComponentProxy( compInfo );
                    trader->registerComponent( ComponentRef( proxy ) );
                }
                //libInfo->ddump();
                trader->registerComponent( ComponentRef( new ComponentProxy( libInfo, lib ) ) );
                delete[] tmp2;
            }
            delete[] rez;
            // end parsing metainfo
            //delete i;
        }
        else
            cerr << "Warning[kore]: \"" << lib->name() << "\" doesn't seem to be a valid kore plugin (the \"" << tmp << "\" symbol is missing)." << endl;
        delete tmp;
    }
    cout << "Done scanLibrary()" << endl;
}

void LibraryLocator::scanDir( const char* path, bool recurse, int flags /* = 0 */ )
{
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
        ComponentRef ref = trader->registeredComponent( "/kore/service/loader" );
        LibraryLoader* loader = dynamic_cast<LibraryLoader*>( ref.instance() );
        if( loader )
        {
#ifndef _WIN32
            struct dirent **namelist;

            int n;

            n = scandir(path, &namelist, 0, alphasort);
            if (n < 0)
                cerr << "scandir" << endl;
            else
            {
                char* buff = new char[200];
                while(n--)
                {
                    if( strcmp( namelist[n]->d_name, "." ) && strcmp( namelist[n]->d_name, ".." ) )
                    {
                        strcpy(buff, path);
                        strcat(buff, "/");
                        strcat(buff, namelist[n]->d_name);
                        cerr << buff << endl;
                        struct stat st;
                        stat( buff, &st );
                        if( S_ISDIR(st.st_mode) )
                        {
                            if( recurse )
                                scanDir( buff, recurse, flags );
                        }
                        else
                        {
                            // Does the name end with ".so" ?
                            int l = strlen( namelist[n]->d_name );
                            if( !strcmp( namelist[n]->d_name + l - 3, ".so" ) )
                            {
                                char* s = loader->fileName2libName(namelist[n]->d_name);
                                //s = loader->fileName2libName(namelist[n]->d_name);
                                cerr << "Scanning: " << s << endl;
                                scanLibrary( s, path, flags );
				//cout << "Done scanning: " << s << endl;
                            }
                        }
                    }
                    free(namelist[n]);
                }
                delete[] buff;
                free(namelist);
            }
#else
            struct _finddata_t c_file;
            long hFile;
            char* buff = new char[1024];
            strcpy(buff, path);
            strcat(buff, "/*.dll");
            cerr << "LOOKING for: " << buff << endl;

            if((hFile = _findfirst(buff, &c_file )) == -1L)
                cerr << "scandir" << endl;
            else
            {
                char* s = loader->fileName2libName(c_file.name);
                cerr << "About to scan: " << path << c_file.name << endl;
                scanLibrary(s, path, flags);

                while(_findnext( hFile, &c_file) == 0)
                {
                    if(c_file.attrib & _A_SUBDIR)
                    {
                        if(recurse)
                        {
                            char* buff = new char[1024];
                            strcpy(buff, path);
                            strcat(buff, "\\");
                            strcat(buff, c_file.name);
                            scanDir(buff, recurse, flags);
                            delete[] buff;
                        }
                    }
                    else
                    {
                        char* s = loader->fileName2libName(c_file.name);
                        cerr << "About to scan: " << path << c_file.name << endl;
                        scanLibrary(s, path, flags);
                    }
                }
            }
            _findclose(hFile);
            delete[] buff;
#endif
        }
    }
    // next, try scanning for Python components:
    LibraryLocator* pyLocator = dynamic_cast<LibraryLocator*>(trader->registeredComponent("/kore/service/locator/python").instance());
    if( pyLocator )
    {
        pyLocator->scanDir( path, false, flags );
    }
    else
        cerr << "Python locator not found" << endl;
}
