
#include <kore/metainfo.h>
#include <vector>
#include <iostream>
//#include <hash_map>

using namespace kore;
using namespace std;
#include "config.h"
#ifdef EXT_HASH_MAP
using namespace __gnu_cxx;
#endif

#ifdef KORE_WIN32
// cl.exe+stlport hack: cl.exe hates 'endl' and borks the compilation
#define endl "\n"
#endif

/*#ifdef _WIN32
   struct ltstr
    {
        bool operator()(const char* s1, const char* s2) const
        {
            return strcmp(s1,s2) == -1;
        }
    }; 
    // Win32 STL doesn't have stl::hash_multimap
    typedef multimap<const char*,const char*, ltstr> metainfo_hash_type;
#else*/
    struct eqstr
    {
        bool operator()(const char* s1, const char* s2) const
        {
            return strcmp(s1,s2) == 0;
        }
    };
    typedef hash_multimap<const char*,const char*, hash<const char*>, eqstr> metainfo_hash_type;

struct ltstr
{
    bool operator()(const char* s1, const char* s2) const
    {
        return strcmp( s1, s2 ) < 0;
    }
};
//#endif

typedef set<const char* , ltstr> str_set_type;

class kore::StringMultiMapPrivate
{
public:
    // the list (multimap<key: char*, value: char*>)
    // 
    metainfo_hash_type _avps;
};

StringMultiMap::StringMultiMap()
{
    _d = new StringMultiMapPrivate();
}

StringMultiMap::StringMultiMap( const char* avps[][2] )
{
    _d = new StringMultiMapPrivate();
    insert( avps );
}

StringMultiMap::StringMultiMap( const char*** avps )
{
    _d = new StringMultiMapPrivate();
    insert( avps );
}

StringMultiMap::~StringMultiMap()
{
    delete _d;
}

void StringMultiMap::clear()
{
    _d->_avps.clear();
}

int StringMultiMap::size()
{
    return _d->_avps.size();
}

bool StringMultiMap::isEmpty()
{
    return _d->_avps.empty();
}

void StringMultiMap::insert( const char* key, const char* value )
{
    // _d->_avps.insert( metainfo_hash_type::value_type( strdup(key), strdup(value) ) );
    _d->_avps.insert( metainfo_hash_type::value_type(key,value) );
}

void StringMultiMap::insert( const char* avps[][2] )
{
    for( int i = 0; avps[i][0]; i++ )
        insert( avps[i][0], avps[i][1] );
}

void StringMultiMap::insert( const char*** avps )
{
    for( int i = 0; avps[i][0]; i++ )
        insert( avps[i][0], avps[i][1] );
}

void StringMultiMap::remove( const char* key, const char* value)
{
    pair<metainfo_hash_type::iterator, metainfo_hash_type::iterator> p = _d->_avps.equal_range(key);
    _d->_avps.erase( p.first, p.second );
    /*
    vector<metinfo_hash_type::iterator> rm;
    for( metainfo_hash_type::iterator i = p.first; i != p.second; ++i )
        if( strcmp( i->second, value ) == 0 )
            rm.push_back(i);
    int n = rm.size();
    for( int j = 0; j<n; j++ )
    {
        _d->_avps.erase(rm[j]);
    }
    */
}

void StringMultiMap::removeAll( const char* key )
{
    _d->_avps.erase( key );
    /*
    pair<metainfo_hash_type::iterator, metainfo_hash_type::iterator> p = _d->_avps.equal_range(key);
    vector<metinfo_hash_type::iterator> rm;
    for( metainfo_hash_type::iterator i = p.first; i != p.second; ++i )
        rm.push_back(i);
    int n = rm.size();
    for( int j = 0; j<n; j++ )
    {
        _d->_avps.erase(rm[j]);
    }
    */
}

void StringMultiMap::replace( const char* key, const char* oldval, const char* newval )
{
    remove( key, oldval );
    insert( key, newval );
}

void StringMultiMap::replaceAll( const char* key, const char* value )
{
    removeAll( key );
    insert( key, value );
}

int StringMultiMap::count( const char* key )
{
    return _d->_avps.count( key );
}

const char* StringMultiMap::find( const char* key )
{
//	cerr << "StringMultiMap::find()"<< endl;
    pair<metainfo_hash_type::iterator, metainfo_hash_type::iterator> p = _d->_avps.equal_range(key);
    vector<metainfo_hash_type::iterator> rm;
    for( metainfo_hash_type::iterator i = p.first; i != p.second; ++i )
        return i->second;
    return 0;
}

const char** StringMultiMap::findAll( const char* key )
{
    pair<metainfo_hash_type::iterator, metainfo_hash_type::iterator> p = _d->_avps.equal_range(key);
    vector<metainfo_hash_type::iterator> rm;
    for( metainfo_hash_type::iterator i = p.first; i != p.second; ++i )
        rm.push_back(i);
    int n = rm.size();
    const char** ret = new const char*[n+1];
    int j;
    for( j = 0; j<n; j++ )
    {
        ret[j] = rm[j]->second;
    }
    ret[j] = 0;
    return ret;
}

const char** StringMultiMap::keys()
{
    str_set_type temp;
    for( metainfo_hash_type::iterator i = _d->_avps.begin(); i!= _d->_avps.end(); ++i )
        temp.insert( i->first );
    const char** rez = new const char*[temp.size()+1];
    int k = 0;
    for( str_set_type::iterator j = temp.begin(); j!= temp.end(); ++j )
        rez[k++] = *j;
    rez[k] = 0;
    return rez;
}

const char*** StringMultiMap::elements()
{
    int n = size();
    const char*** ret = new const char**[n+1];
    int j = 0;
    for( metainfo_hash_type::iterator i = _d->_avps.begin(); i != _d->_avps.end(); ++i )
    {
        ret[j] = new const char*[2];
        ret[j][0] = i->first;
        ret[j][1] = i->second;
        j++;
    }
    ret[j] = new const char*[2];
    ret[j][0] = ret[j][1] = 0;
    return ret;
}

void StringMultiMap::ddump()
{
    cerr << "StringMultiMap::ddump()" << "\n";
    const char*** elem = elements();
    for( int i = 0; elem[i][0]; i++ )
        cerr << ((char*) elem[i][0]) << "=" << elem[i][1] << endl;
    delete[] elem;
    cerr << "------------------------------ ddump()" << endl;
}

MetaInfo::MetaInfo()
{
}

MetaInfo::MetaInfo( const char* avps[][2] ) : StringMultiMap( avps )
{
	cerr << "MetaInfo created" << endl;
}

MetaInfo::MetaInfo( const char*** avps ) : StringMultiMap( avps )
{
}

MetaInfo::~MetaInfo()
{
}

