
#include "pyliblocator.h"

#include <iostream.h>
#include <string.h>
#ifndef _WIN32
#include <dirent.h>
#else
#include <io.h>
#endif
#include <sys/stat.h>

#include <Python.h>
#include <compile.h>

using namespace pykore;

PyLibLocator::PyLibLocator()
{
    cerr << "PyLibLocator created" << endl;
}

PyLibLocator::~PyLibLocator()
{
    cerr << "PyLibLocator destroyed" << endl;
}

void PyLibLocator::scanLibrary( const char* name, const char* path, int /* flags */ )
{
    if( !Py_IsInitialized() )
    {
        cerr << "Initializing threads" << endl;
        PyEval_InitThreads();
        cerr << "Initializing the interpreter" << endl;
        Py_Initialize();
        cerr << "Done initialization" << endl;
    }
    else
        PyEval_AcquireLock();
    if( !name )
    {
        cerr << "Warning[kore]: python plugin name is NULL in PyLibLocator::scanLibrary( const char*, const char*, int )." << endl;
        PyEval_ReleaseLock();
        return;
    }
    if( !path )
    {
        cerr << "Warning[kore]: python plugin path is NULL in PyLibLocator::scanLibrary( const char*, const char*, int ). Loading python plugins with no path is not implemented yet." << endl;
        PyEval_ReleaseLock();
        return;
    }
    int len = strlen( path ) + 1 + strlen( name ) + 4 + 1;
    char* fname = new char[len];
    if( !fname )
    {
        cerr << "Warning[kore]: could not allocate " << len << " bytes in PyLibLocator::scanLibrary( const char*, const char*, int ). Giving up." << endl;
        PyEval_ReleaseLock();
        return;
    }
    strcpy( fname, path );
    strcat( fname, "/" );
    strcat( fname, name );
    strcat( fname, ".py" );
    fname[len-2] = 0;
    fname[len-1] = 0;
    cerr << "trying to open python plugin: " << fname << endl;
    FILE* fd = fopen(fname,"r");
    // try to open the .py file
    if( !fd )
    {
        // now try .pyc
        fname[len-2]='c';
        fd = fopen(fname,"r");
        if( !fd )
        {
            // finally, try .pyo
            fname[len-2]='o';
            fd = fopen(fname,"r");
            if( !fd )
            {
                cerr << "Warning[kore]: could not open python plugin: " << name << " . Possible cause:" << endl;
                perror( NULL );
                delete fname;
                PyEval_ReleaseLock();
                return;
            }
        }
    }
    //PyRun_SimpleFile( fd, fname );
    cerr << "parsing python plugin " << fname << endl;
    struct _node* nd = PyParser_SimpleParseFile( fd, fname, Py_file_input );
    if( !nd || PyErr_Occurred() )
    {
        cerr << "Warning[kore]: parsing python plugin failed. ";
        PyObject* err = PyErr_Occurred();
        if( !err )
        {
            cerr << "No details available." << endl;
        }
        else
        {
            cerr << "Traceback follows:" << endl;
            PyErr_Print();
        }
        fclose( fd );
        delete fname;
        PyEval_ReleaseLock();
        return;
    }
    cerr << "compiling python plugin" << endl;
    PyObject* code = (PyObject*)PyNode_Compile( nd, fname );
    if( !code || PyErr_Occurred() )
    {
        cerr << "Warning[kore]: compiling python plugin failed. ";
        PyObject* err = PyErr_Occurred();
        if( !err )
        {
            cerr << "No details available." << endl;
        }
        else
        {
            cerr << "Traceback follows:" << endl;
            PyErr_Print();
        }
        fclose( fd );
        delete fname;
        PyEval_ReleaseLock();
        return;
    }
    cerr << "importing python plugin" << endl;
    PyObject* module = PyImport_ExecCodeModule( const_cast<char*>(name), code );
    cerr << "done..." << endl;
    if( !module )
    {
        cerr << "Warning[kore]: importing python plugin failed. ";
        PyObject* err = PyErr_Occurred();
        if( !err )
        {
            cerr << "No details available." << endl;
        }
        else
        {
            cerr << "Traceback follows:" << endl;
            PyErr_Print();
        }
        fclose( fd );
        delete fname;
        PyEval_ReleaseLock();
        return;
    }
    Py_INCREF( module );
    cerr << "python plugin imported" << endl;
    fclose( fd );
    PyEval_ReleaseLock();
    delete fname;
}

void PyLibLocator::scanDir( const char* path, bool recurse, int flags )
{
/*
    ComponentTrader* trader = dynamic_cast<ComponentTrader*>(ComponentTrader::traderRef().instance());
    if( trader )
    {
        ComponentRef ref = trader->registeredComponent( "/kore/service/loader" );
        LibraryLoader* loader = dynamic_cast<LibraryLoader*>( ref.instance() );
        if( loader )
        {
*/
#ifndef _WIN32
            struct dirent **namelist;

            int n;

            n = scandir(path, &namelist, 0, alphasort);
            if (n < 0)
                cerr << "scandir" << endl;
            else
            {
                char* buff = new char[200];
                while(n--)
                {
                    if( strcmp( namelist[n]->d_name, "." ) && strcmp( namelist[n]->d_name, ".." ) )
                    {
                        strcpy(buff, path);
                        strcat(buff, "/");
                        strcat(buff, namelist[n]->d_name);
                        //cout << buff << endl;
                        struct stat st;
                        stat( buff, &st );
                        if( S_ISDIR(st.st_mode) )
                        {
                            if( recurse )
                                scanDir( buff, recurse, flags );
                        }
                        else
                        {
                            // Does the name end with ".py" ?
                            int l = strlen( namelist[n]->d_name );
                            if( !strcmp( namelist[n]->d_name + l - 3, ".py" ) )
                            {
                                //char* s = loader->fileName2libName(namelist[n]->d_name);
                                char* s = new char[l+1];
                                strcpy( s, namelist[n]->d_name );
                                s[ l - 3 ] = 0;
                                //s = loader->fileName2libName(namelist[n]->d_name);
                                //cout << "Scanning: " << s << endl;
                                scanLibrary( s, path, flags );
                                delete[] s;
                            }
                            else if( !strcmp( namelist[n]->d_name + l - 4, ".pyc" ) )
                            {
                                //char* s = loader->fileName2libName(namelist[n]->d_name);
                                char* s = new char[l+1];
                                strcpy( s, namelist[n]->d_name );
                                s[ l - 4 ] = 0;
                                //s = loader->fileName2libName(namelist[n]->d_name);
                                //cout << "Scanning: " << s << endl;
                                scanLibrary( s, path, flags );
                                delete[] s;
                            }
                            if( !strcmp( namelist[n]->d_name + l - 4, ".pyo" ) )
                            {
                                //char* s = loader->fileName2libName(namelist[n]->d_name);
                                char* s = new char[l+1];
                                strcpy( s, namelist[n]->d_name );
                                s[ l - 4 ] = 0;
                                //s = loader->fileName2libName(namelist[n]->d_name);
                                //cout << "Scanning: " << s << endl;
                                scanLibrary( s, path, flags );
                                delete[] s;
                            }
                        }
                    }
                    free(namelist[n]);
                }
                delete[] buff;
                free(namelist);
            }
#else
            struct _finddata_t c_file;
            long hFile;
            int moreFiles = !0;
            char* buff = new char[1024];
            strcpy(buff, path);
            strcat(buff, "\\*.*");
            if((hFile = _findfirst(buff, &c_file )) == -1L)
            {
                cerr << "scandir" << endl;
                moreFiles = 0;
            }
            while( moreFiles )
            {
                if(c_file.attrib & _A_SUBDIR)
                {
                    if(recurse)
                    {
                        char* buff = new char[1024];
                        strcpy(buff, path);
                        strcat(buff, "\\");
                        strcat(buff, c_file.name);
                        scanDir(buff, recurse, flags);
                        delete[] buff;
                    }
                }
                else
                {
                    /*
                    char* s = loader->fileName2libName(c_file.name);
                    int l = strlen( c_file.name );
                    char* s = new char[l+1];
                    strcpy( s, c_file._name );
                    s[ l - 3 ] = 0;
                    scanLibrary(s, path, flags);
                    delete[] s;
                    */
                    // Does the name end with ".py" ?
                    int l = strlen( c_file.name );
                    if( !strcmp( c_file.name + l - 3, ".py" ) )
                    {
                        //char* s = loader->fileName2libName(c_file.name);
                        char* s = new char[l+1];
                        strcpy( s, c_file.name );
                        s[ l - 3 ] = 0;
                        //s = loader->fileName2libName(c_file.name);
                        //cout << "Scanning: " << s << endl;
                        scanLibrary( s, path, flags );
                        delete[] s;
                    }
                    else if( !strcmp( c_file.name + l - 4, ".pyc" ) )
                    {
                        //char* s = loader->fileName2libName(c_file.name);
                        char* s = new char[l+1];
                        strcpy( s, c_file.name );
                        s[ l - 4 ] = 0;
                        //s = loader->fileName2libName(c_file.name);
                        //cout << "Scanning: " << s << endl;
                        scanLibrary( s, path, flags );
                        delete[] s;
                    }
                    if( !strcmp( c_file.name + l - 4, ".pyo" ) )
                    {
                        //char* s = loader->fileName2libName(c_file.name);
                        char* s = new char[l+1];
                        strcpy( s, c_file.name );
                        s[ l - 4 ] = 0;
                        //s = loader->fileName2libName(c_file.name);
                        //cout << "Scanning: " << s << endl;
                        scanLibrary( s, path, flags );
                        delete[] s;
                    }
                }
                moreFiles = !_findnext( hFile, &c_file);
            }
/*
            if((hFile = _findfirst(buff, &c_file )) == -1L)
                cerr << "scandir" << endl;
            else
            {
                char* s = loader->fileName2libName(c_file.name);
                scanLibrary(s, path, flags);

                while(_findnext( hFile, &c_file) == 0)
                {
                    if(c_file.attrib & _A_SUBDIR)
                    {
                        if(recurse)
                        {
                            char* buff = new char[1024];
                            strcpy(buff, path);
                            strcat(buff, "\\");
                            strcat(buff, c_file.name);
                            scanDir(buff, recurse, flags);
                            delete[] buff;
                        }
                    }
                    else
                    {
                        char* s = loader->fileName2libName(c_file.name);
                        int l = strlen( c_file.name );
                        char* s = new char[l+1];
                        strcpy( s, c_file._name );
                        s[ l - 3 ] = 0;
                        scanLibrary(s, path, flags);
                        delete[] s;
                    }
                }
            }
*/
            _findclose(hFile);
            delete[] buff;
#endif
/*
        }
    }
*/
}
